/*
 * Copyright (c) 2004 Pam Betten, Nathan Edwards, Adam Goforth, 
 *     Jim Laing, Shahzad Qureshi
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

package eShopper;

import java.text.SimpleDateFormat;
import java.util.*;
import javax.swing.table.AbstractTableModel;


/**
 * This is a day selection class primarily for the add item window. The trickiest parts of
 * this code were originally written way back when we had a calendar tab. This is
 * no doubt the most user-friendly way to enter a day, I think.
 * 
 * @author Jim Laing
 */
class eShopperCalendar extends AbstractTableModel {
	private GregorianCalendar cal;
	private String[] headings;  
	private String[][] entries;
	
	/**
	 * constructor
	 */
	public eShopperCalendar() {
		cal = new GregorianCalendar();	 // set to current day/time
		headings = new String[] {"S", "M", "T", "W", "T", "F", "S"};
		updateEntries();
	} 
	
	/**
	 * builds the entries table for the current month setting of 
	 * the internal calendar (orig. current month/day
	 *
	 */
	public void updateEntries() {
		entries = new String[6][7];  
		int origMonth = cal.get(Calendar.MONTH);
		
		// I couldn't find anything to get the day a month starts on,
		// so this does it by taking a numeric representation of the first date
		// in the month to start the array on and then walking through the month 
		// day by day to create the other entries until it is no longer in the 
		// month it started in 	
		cal.set(Calendar.DATE, 1); // start at one
		for (int i=1, j=0, k; cal.get(Calendar.MONTH) == origMonth; cal.set(Calendar.DATE, ++i)) { 
			k = cal.get(Calendar.DAY_OF_WEEK); 									
			entries[j][k-1] = (new Integer(i)).toString();
			if(k%7 == 0) j++; 																
		}	
		// put us back in the month we started
		cal.add(Calendar.DATE, -1);
		fireTableDataChanged();
	}
	
	/**
	 * Advances the calendar to the next month and updates the calendar table entries
	 *
	 */
	public void nextMonth() {
		cal.add(Calendar.MONTH, 1);
		updateEntries();
	}
	
	/**
	 * sets the calendar to the previous month and updates the calendar table entries
	 *
	 */
	public void previousMonth() {
		cal.add(Calendar.MONTH, -1);
		updateEntries();
	}

	/**
	 * get a string representation of the current month
	 * @return a string of the format String Month Year (e.g. July 2004)
	 */
	public String getMonthString() {
		SimpleDateFormat mmyyyy = new SimpleDateFormat(" MMMM yyyy ");
		return mmyyyy.format(cal.getTime());
	}
	
	/**
	 * Retrives a database-compatible string of the desired date
	 * @param row the table row desired
	 * @param col the table column desired
	 * @return a database-compatible string representation of the date, (e.g '2004-07-04') 
	 */
	public String getDateStringAt(int row, int col) {
		try{
			// holy data types, batman - get an int from a string
			int i = (new Integer((String) getValueAt(row, col))).intValue();
			cal.set(Calendar.DATE, i);
			SimpleDateFormat yyyymmdd = new SimpleDateFormat("''yyyy-MM-dd''");
			return yyyymmdd.format(cal.getTime());
		// if there is no int, that means no selection was made
		} catch(NumberFormatException e) { return null; }
	}
	
	/**
	 * Retrives a label-compatible string of the desired date
	 * @param row the table row desired
	 * @param col the table column desired
	 * @return a database-compatible string representation of the date, (e.g '2004-07-04') 
	 */
	public String getLabelStringAt(int row, int col) {
		try{
			// holy data types, batman - get an int from a string
			int i = (new Integer((String) getValueAt(row, col))).intValue();
			cal.set(Calendar.DATE, i);
			SimpleDateFormat yyyymmdd = new SimpleDateFormat("MMM d yyyy");
			return yyyymmdd.format(cal.getTime());
		// if there is no int, that means no selection was made
		} catch(NumberFormatException e) { return "No date"; }
	}
	
	/**
	 * Retrieves the Date object of the desired date
	 * @param row the table row desired
	 * @param col the table column desired
	 * @return a java.sql.Date representation of the date at that column and row
	 */
	public java.sql.Date getDateAt(int row, int col) {
		try{
			// holy data types, batman - get an int from a string
			int i = (new Integer((String) getValueAt(row, col))).intValue();
			cal.set(Calendar.DATE, i);
			return new java.sql.Date(cal.getTime().getTime());
		// if there is no int, that means no selection was made
		} catch(NumberFormatException e) { return null; }
	}
	
	// these functions must be implemented in any AbstractTableModel class
	/**
	 * @return an integer representing the number of columns in the calendar
	 */
	public int getColumnCount() { 
		return headings.length;
	 }

	/**
	 * @return an integer representing the number of rows in the calendar
	 */
	public int getRowCount() { 
		return entries.length;
	}

	/**
	 * @param the desired column
	 * @return the string representation of the column name
	 */
	public String getColumnName(int col) {
		 return headings[col];
	}

	/**
	 * @param row the table row desired
	 * @param col the table column desired
	 * @return the object in the calendar at that location
	 */
	public Object getValueAt(int row, int col) {
		 // a little hack so invalid values come back as a cell with
		 // no value in it - see getDateStringAt and addFooToList() in AddItemWindow 
		 if(row < 0 || col < 0) {
		 	row = 5;
		 	col = 6;
		 }
		 return entries[row][col]; 
	} 
} 