package eShopper;

import java.sql.Date;
import java.sql.SQLException;
import java.sql.*;
import javax.swing.JTree;

/*
 * Created on Apr 26, 2004
 */

/**
 * eShopperList is a subclass of JTree which creates the single
 * JTree from the tree object. It also passes on a request to change
 * the tree view
 * 
 * @author Jim Laing
 */
public class eShopperList extends JTree {
	
	private eShopperTree tree;
	private eShopperDB db;
	private int listID;
	private expansionListener expanded;
	
	/**
	 * Builds a new eShopperList
	 * @param listID The database ID of the list
	 * @param view The initial view. Corresponds to the DATE, RECIPE, and ITEM static
	 * values found in eShopperRoot
	 * @param db The eShopperDB database which the list is built from
	 */
	public eShopperList(int listID, int view, eShopperDB db) {
		super((new eShopperTree(view, listID, db)));
		this.tree = (eShopperTree) this.getModel();
		this.db = db;
		if(!exists(listID)) {
			create(listID); 
			setListID(listID);
		}
		this.listID = listID;
		expanded = new expansionListener();
		this.addTreeExpansionListener(expanded);
	}
	
	/**
	 * 
	 * @return An eShopperDB object which is the current database. 
	 * This is the database which the list is built from. 
	 */
	public eShopperDB getDB() { return db;	}
	
	/**
	 * Sets the current view. Views correspond to the
	 * static values found in the eShopperRoot class: DATE, RECIPE, and ITEM.
	 * @param view The desired view for the tree
	 */
	public void setView(int view) { 
		tree.setView(view);
		for(int i = 0; i < expanded.size(); i++) this.expandPath(expanded.getPathAt(i)); 
	}
	
	/**
	 * 
	 * @return An int representing the current view. Views correspond to the
	 * static values found in the eShopperRoot class: DATE, RECIPE, and ITEM
	 */
	public int getView() { 
		return ((eShopperRoot) tree.getRoot()).getView(); 
	}
	
	/**
	 * Updates the tree model. Used after a change in the database data.
	 *
	 */
	public void update() { 
		// rebuild the model when the database changes
		tree = new eShopperTree(getView(), listID, db);
		this.setModel(tree);
	}
	
	/**
	 * Changes the list to the specified ID
	 * @param newListID The database ID of the new list
	 */
	public void setListID(int newListID) { 
		this.listID = newListID;
		update();
	}	
	
	/**
	 * 
	 * @return The database ID of the current list
	 */
	public int getListID() { return listID; }
	
	
	/**
	 * 
	 * @return A String containing the name of the current list
	 */
	public String getListName() { return tree.getRoot().toString(); }
	
	/**
	 * This returns the list's owner.
	 * 
	 * @return The owner of the list.
	 */
	public String getListOwner() {
		try{
			String q = "SELECT Owner FROM List WHERE ID = " + listID;
			ResultSet rs = db.query(q);
			if (rs.next()){
				return rs.getString("OWNER");
			}
		}
		catch (SQLException e){
			System.out.println("Error: eShopperList::getListOwner");
			e.printStackTrace();
		}
		return null;
	}
	

	/**
	 * This clears the list (on screen and in the database) of all items.
	 * 
	 */
	public void clear() {
		String q = new String();
		
		// Undo
		//eShopperAction newAction = new eShopperAction();
		//newAction.setAction(1, this);

		try {
			// clear the ingredient list table for this listID
			q = "DELETE FROM INGREDIENTLIST WHERE LISTID = " + listID;
			boolean result = db.update(q);
			// clear the recipe list table for this listID
			q = "DELETE FROM RECIPELIST WHERE LISTID = " + listID;
			result = db.update(q);
		} catch (SQLException e) {
			System.out.println("Error: eShopperList::clear");
			e.printStackTrace();
		}
        this.update();
	}
	
	
	
	/**
	 * This adds a Recipe to this list.
	 * 
	 * @param r The recipe to add to the list.
	 * @param mealString The meal that the Recipe should be associated with.
	 * @param date The date the Recipe should be associated with.
	 */
	public void add(Recipe r, String mealString, Date date) {
		String q = new String();
		String dateString;
		eShopperDate aDate;
		
		if(!exists(listID)) {
			create(listID); 
			setListID(listID);
		}
		
		if(date != null) {
			aDate = new eShopperDate(date, listID, db);
			dateString = aDate.getDBString();
		} else dateString = null;
		
		try {
			q = "INSERT INTO RECIPELIST VALUES(" + null + ", " +
				 r.getID() + ", " + listID + ", '" + mealString + "', " 
				+ dateString + ");";
			db.update(q);
			update();
		} catch (SQLException e) {
			System.out.println("Error: AddItemWindow::addRecipeToList");
			e.printStackTrace();
		}
	}
	
	/**
	 * This adds an ingredient to the list.
	 * 
	 * @param i The Ingredient to be added to the list.
	 * @param mealString The meal that the Ingredient should be associated with.
	 * @param date The date the Ingredient should be associated with.
	 * @param qty The quantity amount of the Ingredient.
	 * @param unitID The unit of measurement the quantity should be measured in.
	 */ 
	public void add(Ingredient i, String mealString, Date date, double qty, int unitID) {
		String q = new String();
		String dateString;
		eShopperDate aDate;
		
		if(!exists(listID)) {
			create(listID); 
			setListID(listID);
		}
		
		if(date != null) {
			aDate = new eShopperDate(date, listID, db);
			dateString = aDate.getDBString();
		} else dateString = null;
		
		try {
			q = "INSERT INTO INGREDIENTLIST VALUES(" + null + ", " + i.getID() 
				+ ", " + listID + ", '" + mealString + "', " 
					+ dateString + ", " + qty + ", " + unitID + ");";
			db.update(q);
			update(); // will go away
		} catch (SQLException e) {
			System.out.println("Error: eShopperList::addIngredient");
			e.printStackTrace();
		}
	}
	
	/**
	 * This removes the item(s) currently selected from the list & database.
	 */
	public void remove() {
			
		String q = new String();
		
		// Return if nothing is selected for removal
		if (this.getSelectionCount() == 0)
			return;
		
		// Get the currently selected node object
		eShopperNode n = (eShopperNode) this.getSelectionPath().getLastPathComponent();
		
		// the next three ifs determine what type of node we have, and takes the appropriate action
		if(n instanceof eShopperDate) {
			String theDate = ((eShopperDate) n).getDBString();
			try{
				// remove all ingredients from the list with that date
				q = "DELETE FROM INGREDIENTLIST WHERE DATE = " + theDate + 
					"AND LISTID = " + listID;
				db.update(q);
				// remove all recipes from the list with that date
				q = "DELETE FROM RECIPELIST WHERE DATE = " + theDate +
					"AND LISTID = " + listID;
				db.update(q);
			} catch (SQLException e) {
				System.out.println("Error: eShopperList::remove");
				e.printStackTrace();
			}
		}
		if(n instanceof eShopperRecipe) {
			try {
				// take the recipe out of the recipelist table
				q = "DELETE FROM RECIPELIST WHERE ID = " + ((eShopperRecipe) n).nodeID();
				db.update(q);
			} catch (SQLException e) {
				System.out.println("Error: eShopperList::remove");
				e.printStackTrace();
			}
		}
		if(n instanceof eShopperIngredient) {
			try {
				// remove ingredient from ingredient list
				q = "DELETE FROM INGREDIENTLIST WHERE ID = " + ((eShopperIngredient) n).nodeID();
				db.update(q);
			} catch (SQLException e) {
				System.out.println("Error: eShopperList::remove");
				e.printStackTrace();
			}
		}			
		if(n instanceof eShopperMisc) {
			// if it date view, we need to remove unassociated recipes and ingredients
			if(getView() == eShopperRoot.DATE) {
				try {
					q = "DELETE FROM RECIPELIST WHERE DATE IS NULL AND LISTID = " + this.getListID();
					db.update(q);
					q = "DELETE FROM INGREDIENTLIST WHERE DATE IS NULL AND LISTID = " + this.getListID();
					db.update(q);
				} catch (SQLException e) {
					System.out.println("Error: eShopperList::remove");
					e.printStackTrace();
				}
			}
			// if it is recipe view, other is all the ingredients
			if(getView() == eShopperRoot.RECIPE) {
				try {
					// clear out all the ingredients
					q = "DELETE FROM INGREDIENTLIST WHERE LISTID = " + this.getListID();
					db.update(q);
				} catch (SQLException e) {
					System.out.println("Error: eShopperList::remove");
					e.printStackTrace();
				}
			}
		}
		
		this.update();
	}
	
	private boolean exists(int id) {
		try {
			ResultSet rs =db.query("SELECT * FROM LIST WHERE ID = " + id);
			return rs.next();
		} catch (SQLException e) {
			System.out.println("eShopperList::exists");
			e.printStackTrace();
		}
		return false;
	}
	
	private void create(int id) {
		try {
			db.query("INSERT INTO LIST VALUES(" + id + ", 'nobody', 'Untitled " + id + "')");
		} catch (SQLException e) {
			System.out.println("eShopperList::create");
			e.printStackTrace();
		}
	}
}

