/*
 * Created on May 12, 2004
 *
 * Copyright (c) 2004 Pam Betten, Nathan Edwards, Adam Goforth, 
 *     Jim Laing, Shahzad Qureshi
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

package eShopper;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.zip.DataFormatException;

/**
 * This class implements units and unit conversion for the Ingredients.
 * 
 * @author Jim Laing
 *
 */
public class eShopperUnit {

	private int unitID;
	private eShopperDB db;
	private float conversionFactor;
	
	/**
	 * constructor
	 * @param aUnitID the database id of the desired unit
	 * @param db the database
	 */
	public eShopperUnit(int aUnitID, eShopperDB db) {
		this.db = db;
		if(aUnitID != 0) { 
			this.unitID = aUnitID;
		} else this.unitID = getDefaultUnit();
		try {
			ResultSet rs = db.query("SELECT CONVERTFACTOR FROM UNIT WHERE ID = " +unitID);
			if(rs.next()) conversionFactor = rs.getFloat("CONVERTFACTOR");
			else conversionFactor = 1;
		} catch (SQLException e){
			System.out.println("Error: eShopperUnit::eShopperUnit");
			e.printStackTrace();
		}
	}
	
	/**
	 * Gets the abbrviation of a string, for use in various combo boxes
	 * @return a string representation of the unit
	 */
	public String toString() { return getAbbrvString(); }
	

	/**
	 * This returns the unit's ID.
	 * @return The ID of the unit.
	 */
	public int getID() { return unitID; }
	
	/**
	 * Converts a value expressed in the unit type's base unit to a value
	 * expressed in this unit's unit
	 * @param value a value expressed in the unit types base unit
	 * @return the value expressed in the unit's unit 
	 */
	public float convertFromBase(float value) { return value/conversionFactor; }
	
	/**
	 * Converts a value to this unit's base unit
	 * @param value a value expessed in this unit's unit
	 * @return a value expressed in this unit's base unit
	 */
	public float convertToBase(float value) { return value*conversionFactor; }
	
	/**
	 * Expresses a value of the same type in this unit
	 * @param value a value expressed in the original unit
	 * @param origUnit the original unit
	 * @return the value as expressed in this object's unit
	 * @throws DataFormatException throws exception when the units are not the same type
	 */
	public float express(float value, eShopperUnit origUnit) throws DataFormatException {
		if(isSameType(origUnit)) 
			return this.convertFromBase(origUnit.convertToBase(value));
		else throw(new DataFormatException());
	}	
	
	/**
	 * Returns the set abbreviation for this unit
	 * @return a string representation of this unit's abbreviation
	 */
	public String getAbbrvString() {
		try {
			ResultSet rs = db.query("SELECT ABBRV FROM UNIT WHERE ID = " +unitID);
			rs.next();
			return rs.getString("ABBRV");
		} catch (SQLException e){
			System.out.println("Error: eShopperUnit::getAbbrvString");
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * Compares the relative size of a unit to this unit
	 * @param aUnit a unit to compare to
	 * @return true if passed unit is smaller than this unit. False otherwise
	 * @throws DataFormatException if the units are not the same type
	 */
	public boolean isLargerThan(eShopperUnit aUnit) throws DataFormatException {
		if(isSameType(aUnit)) return conversionFactor > aUnit.convertToBase(1);
		else throw(new DataFormatException());
	}
	
	/**
	 * Compares the type of units to see if they are combinable. As of release, there
	 * are four types of units: mass, volume, count, and package but this can be
	 * arbitrarily changed according to the values found in the database.
	 * @param aUnit the unit in question
	 * @return true in aUnit is the same type as this unit
	 */
	public boolean isSameType(eShopperUnit aUnit) {
		return aUnit.getType().equals(this.getType());
	}
	
	/**
	 * Determines whether this unit is set as authoritative. Authoritative units
	 * should be more genereal than units of the same type (e.g. package is more 
	 * general than bag or can and hence it is the authoritative unit of type package)
	 * @return true if the unit is set as authoritative, false otherwise
	 */
	public boolean isAuth() {
		try {
			ResultSet rs = db.query("SELECT AUTH FROM UNIT WHERE ID = " +unitID);
			rs.next();
			return rs.getBoolean("AUTH");
		} catch (SQLException e){
			System.out.println("Error: eShopperUnit::isAuth");
			e.printStackTrace();
		}
		return false;
	}
		
	/**
	 * The type of this unit. As of writing, package, count, mass, and volume are
	 * all types of units
	 * @return a string representation of the type of this unit
	 */
	public String getType() {
		try {
			ResultSet rs = db.query("SELECT TYPE FROM UNIT WHERE ID = " +unitID);
			rs.next();
			return rs.getString("TYPE");
		} catch (SQLException e){
			System.out.println("Error: eShopperUnit::getType");
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * The name of this unit
	 * @return a string representing the full name of this unit
	 */
	public String getName() {
		try {
			ResultSet rs = db.query("SELECT NAME FROM UNIT WHERE ID = " +unitID);
			rs.next();
			return rs.getString("NAME");
		} catch (SQLException e){
			System.out.println("Error: eShopperUnit::getName");
			e.printStackTrace();
		}
		return null;
	} 
	
	/**
	 * Gets the default unit from the database
	 * @return an int which is the unitID of the default unit
	 */
	private int getDefaultUnit() {
		try {
			ResultSet rs = db.query("SELECT VALUE FROM CONFIGURE WHERE NAME ='default unit'");
			rs.next();
			String def = rs.getString("VALUE");
			rs = db.query("SELECT ID FROM UNIT WHERE NAME = '" + def + "'");
			rs.next();
			return rs.getInt("ID");
		} catch (SQLException e) {
			System.out.println("Error: eShopperIngredient::eShopperIngredient");
			e.printStackTrace();
		}
		return 1;
	}
}
